
"""Basic objects for the link demo."""

import string, pickle, urlparse
from quixote.html import htmltext

class Category:
    """A category in the tree of links.
    
    Instance attributes:
      id : string
        ID generated from title.
      title : string
        Human-readable title.
      categories : [Category]
        Subcategories contained in this category.
      links : [Link]
        Links in this category.
    """

    def __init__ (self, title):
        self.title = title
        self.id = _generate_id(title)
        self.categories = []
        self.links = []

    def get_category (self, id):
        """(id:string) : Category
        Return the subcategory with the given ID, or None if there's no match.
        """
        for subcat in self.categories:
            if subcat.id == id:
                return subcat
        return None

    def add_link (self, link):
        """(link:Link)
        Add the link to this category, maintaining the list of links
        in sorted order.
        """
        self.links.append(link)
        def c (link1, link2):
            return cmp(link1.title.lower(), link2.title.lower())
        self.links.sort(c)

    def add_category (self, category):
        """(category:Category)
        Add the category as a subdirectory of this category,
        maintaining the list of subcategories in sorted order.
        """
        self.categories.append(category)
        def c (category1, category2):
            return cmp(category1.title.lower(), category2.title.lower())
        self.categories.sort(c)

    def as_html (self, parent_url):
        """(parent_url:string) : htmltext
        Return an HTML representation of a link to a subcategory.
        """
        self_url = urlparse.urljoin(parent_url, self.id + '/')
        s = htmltext("<a href='%s'>%s</a>") % (self_url, self.title)
        return s

class Link:
    """A single link.

    Instance attributes:
      title : string
        Title, to be used as link text.
      url : string
        URL of link.
      description : string
        Additional text describing the link.
    """
    def __init__ (self, title, url, description=""):
        self.title = title
        self.url = url
        self.description = description

    def as_html (self):
        """() : htmltext
        Return an HTML representation of the link.
        """
        s = htmltext("<a href='%s'>%s</a>") % (self.url, self.title)
        if self.description:
            s += ': ' + self.description
        return s

def _generate_id (title):
    s = title.lower()
    s = filter(lambda ch: ch in string.letters + string.digits,
               s)
    s = ''.join(s)
    s = '_'.join(s.split())
    return s
    
def save (filename, root):
    """save(str, Category)
    Saves a tree into the specified file.
    """
    f = file(filename, 'wb')
    pickle.dump(root, f)
    f.close()
    
def load (filename):
    """load(str) : Category
    Unpickles a tree from the given file and returns the root.
    """
    f = file(filename, 'rb')
    root = pickle.load(f)
    f.close()
    return root

def parse_opml (filename):
    """parse_opml(filename) : Category
    Parses the contents of the specified file as OPML and 
    returns the root of the resulting tree.
    """
